/*
 * @(#)NodeList.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

import java.util.ArrayList;

/**
 * A list of data tree nodes. 
 * NodeList.java
 * Created: Fri Aug 29 16:15:52 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.3 $
 */
class DLpairList extends ArrayList<DLpair>
{
	public DLpair getValue(int i) {
		return this.get(i);
	}

} // DLpairList

class IndexList extends ArrayList<Integer>
{
	public void add(int index) {
		this.add(new Integer(index));
	}

	public int getValue(int i) {
		Integer iob = (Integer)this.get(i);
		if (iob == null) {
			return -2; //undefined
		} else {
			return iob.intValue();
		}
	}

	public Object pop() {
		Object obj = this.get(this.size() - 1);
		this.remove(this.size() - 1);
		return obj;
	}

	public boolean equals(IndexList B) {
		IndexList A = this;
		if (A.size() != B.size())
			return false;
		for (int i = 0; i < A.size(); i++) {
			if (A.getValue(i) != B.getValue(i)) {
				System.out.println("@debug: IndexList.equals: disagree at "+i+":"+
								   A.getValue(i)+"!="+B.getValue(i));
				return false;
			}
		}
		return true;
	}

	public String getString() {
		StringBuffer strBuf = new StringBuffer();
		for (int i = 0; i < this.size(); i++) {
			strBuf.append(this.getValue(i)+" ");
		}
		return strBuf.toString(); 
	}

} // indexList

