package unot;

import java.io.*;
import java.util.*;

/**
 * A template for a writer class.
 * Each object of this class has the only method write(Pattern).
 */
abstract public class PatternWriter {

	/** $B%Q%?!<%s$N=PNO<jB3$-$r@Z$jBX$($k(B */
	public static final int WRITER_DEPTH  = 0; 
	public static final int WRITER_PARENT = 1; 
	public static final int WRITER_TREE   = 2; 
	public static final int WRITER_SERIAL = 3; //UnotPattern only

	//abstract String getName(); 
	static final String writerName = null; 
	public String getName() { return writerName; }

	abstract String writeTree(Pattern pattern);

	//For indentation
	static final String INDENT_STRING = "  ";
	boolean useIndent = false; 

	public void setIndent(boolean flag) {
		this.useIndent = flag; 
	};

	public void putIndent(StringBuffer strBuf, int depth) {
		if (useIndent)
			for (int d = 0; d < depth; d++)
				strBuf.append(INDENT_STRING);
	}

	String write(Pattern pattern) {
		StringBuffer strBuf = new StringBuffer();
		strBuf.append("@pattern ");
		if (pattern.id >= 0) strBuf.append(pattern.id+"\n");
		else strBuf.append("\n");

		//Writing a given tree with a specific TreeWriter
        strBuf.append(writeTree(pattern));

		int patSize = pattern.getCodeSize(); 
		if (patSize >= 0) { //not undef
			strBuf.append("@size "+(patSize)+"\n");
		}

		int rootCount = pattern.getRootCount();
		if (rootCount > -1) { //not undef
			strBuf.append("@rootcount "+(rootCount)+"\n");
		}

		int docCount = pattern.getDocCount();
		if (docCount > -1) { //not undef
			strBuf.append("@doccount  "+(docCount)+"\n");
		}

		NodeList rootList = pattern.getRootList();
		if (rootList != null) {
			strBuf.append("@rootlist "+(rootList.getNodeIdListString())+"\n");
		}

		IndexList docList = pattern.getDocList();
		if (docList != null) {
			strBuf.append("@doclist "+(getIdListString(docList))+"\n");
		}

		IndexList orgRootList = pattern.orgRootList;
		if (orgRootList != null) {
			strBuf.append("@orgRootlist "+orgRootList.getString()+"\n");
		}

		//This cast is dangerous!
		IndexList matchList = pattern.getMatchList();
		if (matchList != null) {
			strBuf.append("@matchlist "+(getIdListString(matchList))+"\n");
		}

		return strBuf.toString();
	}

	public String getIdListString(ArrayList list) {
		StringBuffer strBuf = new StringBuffer();
		for (int i = 0; i < list.size(); i++) {
			Integer val = (Integer) list.get(i);
			strBuf.append(val+" ");
		}
		return strBuf.toString(); 
	}



}


