/*
 * @(#)TreeLoader.java
 *
 * Copyright 2005, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

import java.io.*;
import java.util.*;

/**
 * TreeLoader.java
 *
 * Created: Fri Aug 29 16:15:52 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.2 $
 */
public class TreeLoader
{
    private boolean DEBUG = false;
    //private boolean DEBUG = true;

	/** Ϣʸ񽸹 */
	private DocSet docSet;

	/** ǱޤǼΡɥå */
	StackList right = null; 

    /**
     * Creates a new <code>TreeLoader</code> instance.
     */
    public TreeLoader(DocSet docSet)
    {
		this.docSet = docSet;
		this.right = new StackList(); //initialize the stack
    }

	/*
	 * Basic routienes
	 */

	public void reinitialize() {
		this.right = new StackList(); //initialize the stack
	}

	public void add(DLpair pair) {
		// Create a new node with current infomation of DocSet 
		UnotNode node = this.docSet.createNewNodeCurrent(pair.getLabel());

		// Shrink the rightmost branch to depth - 1 .
		while (this.right.size() > pair.getDepth()) {
			this.right.pop();
		}

		// Set parent 
		UnotNode parent;
		if (this.right.size() <= 0)
			parent = null;
		else 
			parent = (UnotNode)this.right.getTop();
		node.setParentChildRelation(parent);
		//The above routine correctly handles the case when parent = null.

		// Add new node into the right branch
		this.right.push(node);
	}

	public UnotNode getRoot() {
		UnotNode root = (UnotNode)this.right.get(0);
		return root;
	}

	/*
	 * Utilities routienes: A sample
	 */

	UnotNode load(Pattern pattern) {
		//A stack for the nodes on the rightmost branch 
		if (pattern.getCodeSize() <= 0)
			return null; //empty

		this.reinitialize();
		for (int i = 0; i < pattern.getCodeSize(); i++) {
			this.add(pattern.getNodeAt(i));
		}
		UnotNode root = this.getRoot();
		this.docSet.addRoot(root);
		return root;
	}

}//TreeLoader


