/*
 * @(#)Unot.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

import java.util.*;

/**
 * Unot.java
 *
 *
 * Created: Sat Aug 16 03:04:00 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.9 $
 * @see <a href="http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf">
 * Discovering Frequent Substructures in Large Unordered Trees (PDF)</a>
 */
public abstract class Unot
{

    /** Java  DEBUG ͡ */
    public static final boolean DEBUG = true;

    /** ｪλ֤ */
    public static final int EXIT_SUCCESS = 0;

    /** ｪλ֤ */
    public static final int EXIT_FAILURE = 1;

    /** ѥΥ¤ʤȤ򼨤 */
    public static final int MAX_SIZE_NO_LIMIT = 0;

    /** Сʸ*/
    public static final String VERSION_STRING = "Unot $Revision: 1.9 $";

    /** Υ饹Ѥɥȥåȡ󥹥ȥ饯ǻꤹ롥 */
    protected DocSet docset;

    /** ѽХѥΥꥹȤǼѥ󥻥åȡ */
    protected PatSet patset;

    /**  code κ祵ƵߤƤ礭ʥѥʤ
        ǥեȤ MAX_SIZE_NO_LIMIT Ǥꡤ祵̵Ǥ롥 */
    protected int maxCodeSize = MAX_SIZE_NO_LIMIT;

    /** ѥΥǡؤνи */
    protected long numberOfOccurrence;

    /** Ƶؿ expand() ¹Ԥ */
    protected int numberOfExpansion;

	/** ץ󡧥ѥ˺иޤ뤫ɤ */
	public static boolean useRootCount= false; 

	/** ץ󡧥ѥʸиꥹȾޤ뤫ɤ */
	public static boolean useDocCount = false; 

	/** ץ󡧥ѥ˺иꥹȾޤ뤫ɤ */
	public static boolean useRootList = false; 

	/** ץ󡧥ѥʸиꥹȾޤ뤫ɤ */
	public static boolean useDocList = false; 

	/** ץ󡧥Х󥹥åŬѤ뤫ɤ */
	public static boolean useBalanceCheck = true; 

    /**
     * Creates a new <code>Unot</code> instance.
     *
     */
    protected Unot()
    {
        this.patset = new PatSet(null, false);
    }


    /**
     * Creates a new <code>Unot</code> instance.
     *
     * @param docset a <code>DocSet</code> value
     */
    public Unot(DocSet docset)
    {
        if (docset == null)
            throw new IllegalArgumentException("Error: DocSet is null.");

        this.docset = docset;
        this.patset = new PatSet(this.docset);
        this.maxCodeSize = MAX_SIZE_NO_LIMIT;
        this.numberOfOccurrence = 0;
        this.numberOfExpansion = 0;

    } // Unot constructor


    /**
     * 祳ɥꤹ롥
     *
     * @param maxCodeSize an <code>int</code> value
     */
    public void setMaxCodeSize(int maxCodeSize)
    {
        if (maxCodeSize == MAX_SIZE_NO_LIMIT) {
            UnotLog.log("@log Set_MaxCodeSize: No_Limit\n");
        } else {
            UnotLog.log("@log Set_MaxCodeSize: " + maxCodeSize + "\n");
        }

        this.maxCodeSize = maxCodeSize;
    }


    /** ѽڤޥ˥󥰤롥*/
    public PatSet run(double minSupRatio)
    {
        UnotLog.log("@log Minimum_Support_Ratio: " + minSupRatio + "\n");
		int minsup = this.docset.convertMinSupRatioToAbsolute(minSupRatio);
        return run(minsup);
    }

    /** ѽڤޥ˥󥰤롥*/
    public PatSet run(NodeList rootList, double minSupRatio)
    {
        UnotLog.log("@log Minimum_Support_Ratio: " + minSupRatio + "\n");
		int minsup = this.docset.convertMinSupRatioToAbsolute(minSupRatio);
        return run(rootList, minsup);
    }

    /**
     * Executes `UNOT' main algorithm.
     *
     * @param minSup an <code>int</code> value
     * @return a <code>PatSet</code> value
     */
    public PatSet run(int minSup)
    {

        UnotLog.log("@log Minimum_Support_Absolute: " + minSup + "\n");
        if (minSup < 1)
            throw new IllegalArgumentException("Minimum support value is too small.");

        // minSup ʾνи٤ĽиꥹȤΥꥹȤ롥
        OccListTable freqOneNodeList = this.docset.getOccLists(minSup);

        // иꥹȤΥꥹȤΥ󤹤٥ˤ UnotPattern 
		UnotPattern.setMaxLabel(freqOneNodeList.size());

        for (int labelid = 0; labelid < freqOneNodeList.size(); labelid++) {

            // create root node
            UnotPattern pattern = new UnotPattern(labelid);
            OccList occList = freqOneNodeList.getOccList(labelid);
            this.numberOfOccurrence += occList.size();

            // recursion
            expand(pattern, occList, minSup);
        }

        // echo back
        UnotLog.log("@log NumOfExpansion: " + this.numberOfExpansion + "\n");
        UnotLog.log("@log NumOfOccurrence: " + this.numberOfOccurrence + "\n");

        return this.patset;
    }

    /**
     * Executes `UNOT' main algorithm.
     *
     * @param minSup an <code>int</code> value
     * @return a <code>PatSet</code> value
     */
    public PatSet run(NodeList rootList, int minSup)
    {

        UnotLog.log("@log Minimum_Support_Absolute: " + minSup + "\n");
        if (minSup < 1)
            throw new IllegalArgumentException("Minimum support value is too small.");

        // Ϳ줿ʸΡɥꥹȤ顤иꥹɽۤ롥
		// ʸΡɤλ¹ȤʤΡɤޥ˥󥰤뤳Ȥա
        OccListTable freqOneNodeList = this.docset.getOccLists(rootList);

        // иꥹȤΥꥹȤΥ󤹤٥ˤ UnotPattern 
		UnotPattern.setMaxLabel(freqOneNodeList.size());

        for (int labelid = 0; labelid < freqOneNodeList.size(); labelid++) {

            // create root node
            UnotPattern pattern = new UnotPattern(labelid);
            OccList occList = freqOneNodeList.getOccList(labelid);
            this.numberOfOccurrence += occList.size();

            // recursion
            expand(pattern, occList, minSup);
        }

        // echo back
        UnotLog.log("@log NumOfExpansion: " + this.numberOfExpansion + "\n");
        UnotLog.log("@log NumOfOccurrence: " + this.numberOfOccurrence + "\n");

        return this.patset;
    }

	abstract protected void expand(UnotPattern pattern, OccList occList, int minSup);

	//expand
	//updateocc


    /**
     * Describe <code>isCanonicalOcc</code> method here.
     *
     * @param pattern an <code>UnotPattern</code> value
     * @param occ an <code>Occurrence</code> value
     * @param copyDepth an <code>int</code> value
     * @param topDepth an <code>int</code> value
     * @return a <code>boolean</code> value
     */
    protected boolean isCanonicalOcc(UnotPattern pattern, Occurrence occ, int copyDepth, int topDepth)
    {
        for (int dep = copyDepth; dep < topDepth; dep++) {
            if (pattern.isBalanced(dep)) {
                UnotNode leftNode = (UnotNode) occ.get(pattern.getLeftIndex(dep));
                UnotNode rightNode = (UnotNode) occ.get(pattern.getRightIndex(dep));
                if (leftNode.getNodeId() > rightNode.getNodeId()) {
                    return false;
                }
            }
        }
        return true;
    }

	void maintainPatternInfo(Pattern pattern, OccList occList) {
		NodeList rootList = null; 
		IndexList docList = null; 

		//Note: ȤʤѽХѥ󤬸Ĥä
		if (useRootList) {
			rootList = occList.getRootList(); 
			pattern.setRootList(rootList);
		}

		//Note: ȤʤѽХѥ󤬸Ĥä
		if (useDocList) {
			docList = occList.getDocIdList(); 
			pattern.setDocList(docList);
		}

		//Note: ȤʤѽХѥ󤬸Ĥä
		if (useRootCount) {
			//Tricky: saving redundant computation
			int count = -1; 
			if (rootList != null)
				count = rootList.size();
			else
				count = occList.getRootCount();

			pattern.setRootCount(count);
		}

		//Note: ȤʤѽХѥ󤬸Ĥä
		if (useDocCount) {
			//Tricky: saving redundant computation
			int count = -1; 
			if (docList != null)
				count = docList.size();
			else
				count = occList.getDocCount();

			pattern.setDocCount(occList.getDocCount());
		}

	}

} // Unot

