/*
 * @(#)UnotLog.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

/**
 * UnotLog.java
 *
 *
 * Created: Sat Apr  8 03:04:00 2004
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.3 $
 * @see <a href="http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf">
 * Discovering Frequent Substructures in Large Unordered Trees (PDF)</a>
 */
public class UnotLog
{

    public static final int LOG_DEFAULT = 3;
    public static final int LOG_WARNING = LOG_DEFAULT + 1;
    public static final int LOG_NOTICE = LOG_DEFAULT;
    public static final int LOG_INFO = LOG_DEFAULT - 1;
    public static final int LOG_DEBUG = LOG_DEFAULT - 2;

    /**
     * UnotLog.log() ؿϤ٥롥
     * ǥեȤ LOG_DEFAULT ȤʤäƤ롥
     */
    protected static int logLevel = LOG_DEFAULT;


    /**
     * ٥ <code>loglevel</code> ꤹ롥
     *
     * @param loglevel an <code>int</code> value
     */
    public static final void setLogLevel(int logLevel)
    {
        UnotLog.logLevel = logLevel;
        log("@log Set_LogLevel: " + logLevel + "\n");
    }


    /**
     * ˻ꤵ줿٥뤬ߤΥ٥ʾʤ
     * ΥåϤ롥
     *
     * @param loglevel an <code>int</code> value
     * @param msg a <code>String</code> value
     */
    public static final void log(int logLevel, Object msg)
    {
        if (UnotLog.logLevel <= logLevel) {
            System.out.print(msg);
        }
    }

    /**
     * LOG_DEFAULT ߤΥ٥ʾʤ餽ΥåϤ롥
     *
     * @param msg a <code>String</code> value
     */
    public static final void log(Object msg)
    {
        if (UnotLog.logLevel <= LOG_DEFAULT) {
            System.out.print(msg);
			System.out.flush();
        }
    }

	/** åϤư۾ｪλ */
	public static void exit(String msg) {
		System.out.println("Error: "+msg);
		System.exit(1);
	}


} // UnotLog
