/*
 * @(#)UnotPattern.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

/**
 * UnotPattern.java
 *
 * Υ饹 UNOT ѥΥǡ¤ɽ롥 (trcs216.pdf pp.8 Fig.4)
 *
 * Created: Thu Aug  7 20:34:14 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.7 $
 */
public class UnotPattern extends Pattern 
{
	/*
	 * Static Fields
	 */

	/*
	 * Member Fields
	 */

    /** Infinity Node ɽ static ʥΡɡ
        Infinity Node ȤϤ˻ҶʤȤ򼨤ʼǡ
        ɤΥ٥礭ʽŤ˥٥ͤġ */
    protected static DLpair inftyNode;

    /** Infinity Node ؤβۥǥå
     *  left ޤ right ˤͤС˻ҶʤȤɽ
	 */
    //public static final int INFTY_INDEX = -1;

    /** Infinity Node β depth */
    public static final int INFTY_DEPTH = -1;

    /**
     * Infinity Node 롥
     *
     * @param numOfLabels an <code>int</code> value
     */
    public static void setMaxLabel(int numOfLabels)
    {
        UnotPattern.inftyNode = new DLpair(INFTY_DEPTH, numOfLabels);
    }

    /* Modified by arim@ist */
    /**
     * Infinity Node 롥
     *
     * @param numOfLabels an <code>int</code> value
     */
    public static void initialize(int numOfLabels, DocSet docset)
    {
        UnotPattern.inftyNode = new DLpair(INFTY_DEPTH, numOfLabels);
        UnotPattern.docSet    = docset;
    }

	/** ѥϥ롼ꤹ.
	 *  UnotPatternѤPatternWriterηUnotPattern.
	 */
	public static void setUnotPatternWriter(UnotPatternWriter writer) {
		Pattern.writer = writer;
	};

	/** ѥϥ롼֤.	 
	 *  UnotPatternѤPatternWriter֤ͤηUnotPattern.
	 */
	public static UnotPatternWriter getUnotPatternWriter() {
		return (UnotPatternWriter)Pattern.writer; 
	};

	/*
	 * Members
	 */

    /** (depth, label) Ф */
    private DLpair[] code;

    /** code ˤ뺸ڤؤźǥåͤ */
    private int[] left;

    /** code ˤ뱦ڤؤźǥåͤ */
    //private int[] right; //implemented by class Pattern 

    /** Ǳ޾γƹ⤵ˤ뱦ڤΥɤ
        ڤΥɤȲʸפƤ뤫򼨤ͤ */
    private int[] lcp; // the longest common prefix

    /** ԡ͡ */
    private int copydepth;

//     /** иȤʤΡɤΥꥹȡץ */
//     private NodeList rootList = null;

//     /** иȤʤΡɤΥꥹȡץ */
//     private int rootCount = -1; //undef

    /**
     * Creates a new <code>UnotPattern</code> instance.
     *
     */
    public UnotPattern()
    {
        // do nothing.
    } // UnotPattern constructor


    /**
     * Returns a UnotPattern object that has only root node.
     *
     * @param rootLabel an <code>int</code> value
     */
    public UnotPattern(int rootLabel)
    {
        this.code = new DLpair[] { new DLpair(0, rootLabel) };
        this.left = new int[] { INFTY_INDEX };
        this.right = new int[] { INFTY_INDEX };
        this.lcp = new int[] { 0 };
        this.copydepth = 0;
		this.rootList = null; 
    }


    /**
     * Creates a new <code>UnotPattern</code> instance.
     *
     * @param code a <code>DLpair[]</code> value
     * @param left an <code>int[]</code> value
     * @param right an <code>int[]</code> value
     * @param lcp an <code>int[]</code> value
     * @param copydepth an <code>int</code> value
     */
    public UnotPattern(DLpair[] code, int[] left, int[] right,
                       int[] lcp, int copydepth)
    {
        this.code = code;
        this.left = left;
        this.right = right;
        this.lcp = lcp;
        this.copydepth = copydepth;

    } // UnotPattern constructor

    /**
     * Get the code size (= the number of nodes).
     *
     * @return an <code>size</code> of the pattern
     */
    public int getCodeSize()
    {
        return this.code.length;
    }

    /**
     * Get the depth-label pair for the i-th node. 
     *
     * @return a DLpair <code>(depth, label)</code>
     */
    public DLpair getNodeAt(int i)
    {
        return this.code[i];
    }

    public int[] getRight() { return this.right;  }

    public int[] getLeft() { return this.left;  }

    public int[] getLcp() { return this.lcp;  }

    public int   getCopyDepth() { return this.copydepth;  }

    public NodeList getRootList() { return this.rootList; }

    public int   getRootCount() { return this.rootCount; }

    public void  setRootCount(int rootCount) { this.rootCount = rootCount; }

    public void  setRootNodeList(NodeList list) { this.rootList = list; }

    /**
     * Get the top depth of the rightmost branch.
     *
     * @return an <code>int</code> value
     */
    public int getTopDepth()
    {
        return this.right.length - 1;
    }


    /**
     * Gets the top index of the `code' array of this object.
     *
     * @return an <code>int</code> value
     */
    public int getTopIndex()
    {
        return this.code.length - 1;
    }


    /**
     * Gets the index of the node on the `code' array of this object.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getNodeIndex(int depth)
    {
        return (depth == 0) ? 0 : this.right[depth-1];
    }


    /**
     * Gets the label of the rightmost branch node at the depth.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getNodeLabel(int depth)
    {
        int index = getNodeIndex(depth);

        if (index == INFTY_INDEX) {
            return this.inftyNode.getLabel();
        } else {
            return this.code[index].getLabel();
        }
    }


    /**
     * Gets the index of the depth in the rightmost branch.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getRightIndex(int depth)
    {
        return this.right[depth];
    }


    /**
     * Gets the index of the left node at the depth in the rightmost branch.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getLeftIndex(int depth)
    {
        return this.left[depth];
    }


    /**
     * Distinguish whether the depth node is Infinity Node or not.
     *
     * @param depth an <code>int</code> value
     * @return a <code>boolean</code> value
     */
    public boolean isInftyNode(int depth)
    {
        return (getNodeIndex(depth) == INFTY_INDEX) ? true : false;
    }


    /**
     * Compares the left with the right in the node at the depth.
     *
     * @param depth an <code>int</code> value
     * @return a <code>boolean</code> value
     */
    public boolean isBalanced(int depth)
    {
        if ((this.copydepth <= depth)
            && (this.right[depth] - this.left[depth]) == this.lcp[depth]) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * Gets the disagreement node depth.
     * `disagreement node' is the node of the position j
     * if j <= min(|C(Li)|, |C(Ri)|) and the j-th components of
     * C(Li) and C(Ri) are different pairs.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getDisagreeNodeDepth(int depth)
    {
        if (depth == this.copydepth) {
            return this.code[left[depth] + lcp[depth]].getDepth();
        } else {
            throw new IllegalArgumentException();
        }
    }


    /**
     * Gets the disagreement node label.
     * `disagreement node' is the node of the position j
     * if j <= min(|C(Li)|, |C(Ri)|) and the j-th components of
     * C(Li) and C(Ri) are different pairs.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getDisagreeNodeLabel(int depth)
    {
        if (depth == this.copydepth) {
            return this.code[this.left[depth] + this.lcp[depth]].getLabel();
        } else {
            throw new IllegalArgumentException();
        }
    }


    /**
     * Returns a UnotPattern which is expanded with the Rightmost Expantion.
     *
     * @param oldPattern an <code>UnotPattern</code> value
     * @param newNode a <code>DLpair</code> value
     * @param succCopyDepth an <code>int</code> value
     * @return an <code>UnotPattern</code> value
     */
    public UnotPattern rightmostExpand(UnotPattern oldPattern, DLpair newNode, int succCopyDepth)
    {
        UnotPattern newPattern = new UnotPattern();

        int codeLength = oldPattern.code.length + 1;
        int rightLength = newNode.getDepth() + 1;

        // allocate
        newPattern.code = new DLpair[codeLength];
        newPattern.left = new int[rightLength];
        newPattern.right = new int[rightLength];
        newPattern.lcp = new int[rightLength];

        // copy
        System.arraycopy(oldPattern.code, 0, newPattern.code, 0, codeLength-1);
        System.arraycopy(oldPattern.left, 0, newPattern.left, 0, rightLength-1);
        System.arraycopy(oldPattern.right, 0, newPattern.right, 0, rightLength-1);

        int topIndex = newPattern.getTopIndex();
        int topDepth = newPattern.getTopDepth();

        // add new node
        newPattern.code[topIndex] = newNode;
        newPattern.left[topDepth-1] = oldPattern.right[topDepth-1];
        newPattern.right[topDepth-1] = topIndex;

        // add infinity node
        newPattern.inftyNode = oldPattern.inftyNode;
        newPattern.left[topDepth] = INFTY_INDEX;
        newPattern.right[topDepth] = INFTY_INDEX;

        // copy lcp
        //(lcp[0..copydep] is not used, thus they aren't copied)
        System.arraycopy(oldPattern.lcp, succCopyDepth, // from
                         newPattern.lcp, succCopyDepth, // to
                         topDepth - succCopyDepth);     // length
        newPattern.lcp[topDepth-1] = 0; // reset parent's lcp
        // and update lcp (lcp[copydep..topdep])
        for (int dep = succCopyDepth; dep < topDepth; dep++) {
            if ((newPattern.left[dep] != INFTY_INDEX)
                && (newPattern.code[
                        newPattern.left[dep] + newPattern.lcp[dep]
                    ].equals(newNode))) { // equals <=> (d1, l1) == (d2, l2)
                newPattern.lcp[dep]++;
            }
        }
        newPattern.lcp[topDepth] = 0; // reset top's lcp

        // copydepth
        newPattern.copydepth = succCopyDepth;

        return newPattern;

    } // rightmostExpand


    /**
     * Returns a cloned object.
     *
     * @return an <code>Object</code> value
     * @exception CloneNotSupportedException if an error occurs
     */
    public Object clone() throws CloneNotSupportedException
    {
        UnotPattern newPattern = (UnotPattern) super.clone();

        newPattern.code  = (DLpair[]) this.code.clone();
        newPattern.left  = (int[]) this.left.clone();
        newPattern.right = (int[]) this.right.clone();
        newPattern.lcp = (int[]) this.lcp.clone();

        return (Object) newPattern;
    }


   /**
     * Retrieves a String representation of the object.
     *
     * @return a <code>String</code> representation of this object.
     * @see Object#toString()
     */
    public String toString()
    {
        StringBuffer strBuf = new StringBuffer();

        strBuf.append("[code: ");
        for (int i = 0; i < this.code.length; i++) {
            strBuf.append(this.code[i].toString());
        }

        strBuf.append("\n left: ");
        for (int i = 0; i < this.left.length; i++) {
            if (0 < i) strBuf.append(", ");
            strBuf.append(this.left[i]);
        }

        strBuf.append("\n right: ");
        for (int i = 0; i < this.right.length; i++) {
            if (0 < i) strBuf.append(", ");
            strBuf.append(this.right[i]);
        }

        strBuf.append("\n lcp: ");
        for (int i = 0; i < this.lcp.length; i++) {
            if (0 < i) strBuf.append(", ");
            strBuf.append(this.lcp[i]);
        }

        strBuf.append("\n copydepth: ");
        strBuf.append(this.copydepth);
        strBuf.append("]");

        return strBuf.toString();
    }

    /**
     * Retrieve a tree like String representation of the object.
     *
     * @return a <code>String</code> value
     */
    public String writeToString() {
		return UnotPattern.writer.write(this);
	}

} // UnotPattern

////////////////////////////////////////////////


/**
 * A template for a reader class.
 * Each object of this class has the only method read(InputStream).
 */
abstract class UnotPatternReader {

	abstract UnotPattern read();
}



//EOF
